@@ -36,6 +36,10 @@ using namespace std;
 #define HAVE_FDATASYNC
 #endif
 
+#define READ_TEST_SIZE 204
+#define OPEN_READ_ATTEMPTS 5
+
+
 class ThreadedFileWriter
 {
 public:
@@ -81,7 +85,7 @@ static unsigned safe_write(int fd, const void *data, unsigned sz)
     unsigned tot = 0;
     unsigned errcnt = 0;
 
-    while (tot < sz) 
+    while (tot < sz)
     {
         ret = write(fd, (char *)data + tot, sz - tot);
         if (ret < 0)
@@ -96,14 +100,14 @@ static unsigned safe_write(int fd, const void *data, unsigned sz)
                      "ERROR: file I/O problem in safe_write(), errcnt = %d",
                      errcnt);
             perror(msg);
-            if (errcnt == 3) 
+            if (errcnt == 3)
                 break;
         }
         else
         {
             tot += ret;
         }
-        if (tot < sz) 
+        if (tot < sz)
             usleep(1000);
     }
     return tot;
@@ -130,7 +134,7 @@ ThreadedFileWriter::ThreadedFileWriter(const char *filename,
 
     fd = open(filename, flags, mode);
 
-    if (fd <= 0)
+    if (fd < 0)
     {
         /* oops! */
         VERBOSE(VB_IMPORTANT,"ERROR opening file in ThreadedFileWriter.");
@@ -155,7 +159,8 @@ ThreadedFileWriter::~ThreadedFileWriter()
 
     pthread_join(writer, NULL);
 
-    close(fd);
+    if (fd >= 0)
+        close(fd);
     fd = -1;
 
     if (buf)
@@ -179,10 +184,10 @@ unsigned ThreadedFileWriter::Write(const void *data, unsigned count)
         first = 0;
         usleep(5000);
     }
-   
+
     if (no_writes)
         return 0;
- 
+
     if ((wpos + count) > tfw_buf_size)
     {
         int first_chunk_size = tfw_buf_size - wpos;
@@ -255,16 +260,17 @@ void ThreadedFileWriter::SetWriteBufferMinWriteSize(int newMinSize)
 void ThreadedFileWriter::DiskLoop()
 {
     int size;
+    int written = 0;
     QTime timer;
     timer.start();
 
     while (!in_dtor || BufUsed() > 0)
     {
         size = BufUsed();
-        
-        if ((!in_dtor) &&
-            (!flush) &&
-            ((unsigned)size < tfw_min_write_size))
+
+        if (!size || (!in_dtor && !flush &&
+            (((unsigned)size < tfw_min_write_size) && 
+             ((unsigned)written >= tfw_min_write_size))))
         {
             usleep(500);
             continue;
@@ -295,7 +301,12 @@ void ThreadedFileWriter::DiskLoop()
         {
             size = safe_write(fd, buf+rpos, size);
         }
-
+        
+        if ((unsigned) written < tfw_min_write_size)
+        {
+            written += size;
+        }
+        
         pthread_mutex_lock(&buflock);
         rpos = (rpos + size) % tfw_buf_size;
         pthread_mutex_unlock(&buflock);
@@ -334,6 +345,7 @@ unsigned ThreadedFileWriter::BufFree()
 
 RingBuffer::RingBuffer(const QString &lfilename, bool write, bool usereadahead)
 {
+    int openAttempts = OPEN_READ_ATTEMPTS;
     Init();
 
     startreadahead = usereadahead;
@@ -341,10 +353,13 @@ RingBuffer::RingBuffer(const QString &lfilename, bool write, bool usereadahead)
     normalfile = true;
     filename = (QString)lfilename;
 
+    if (filename.right(4) != ".nuv")
+        openAttempts = 1;
+
     if (write)
     {
         tfw = new ThreadedFileWriter(filename.ascii(),
-                                     O_WRONLY|O_TRUNC|O_CREAT|O_LARGEFILE, 
+                                     O_WRONLY|O_TRUNC|O_CREAT|O_LARGEFILE,
                                      0644);
         writemode = true;
     }
@@ -374,7 +389,40 @@ RingBuffer::RingBuffer(const QString &lfilename, bool write, bool usereadahead)
             is_local = true;
 
         if (is_local)
-            fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE|O_STREAMING);
+        {
+            char buf[READ_TEST_SIZE];
+            while (openAttempts > 0)
+            {
+                int ret;
+                openAttempts--;
+                
+                fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE|O_STREAMING);
+                
+                if (fd2 < 0)
+                {
+                    VERBOSE( VB_IMPORTANT, QString("Could not open %1.  %2 retries remaining.")
+                                           .arg(filename).arg(openAttempts));
+                    usleep(500000);
+                }
+                else
+                {
+                    if ((ret = read(fd2, buf, READ_TEST_SIZE)) != READ_TEST_SIZE)
+                    {
+                        VERBOSE( VB_IMPORTANT, QString("Invalid file handle when opening %1.  "
+                                                       "%2 retries remaining.")
+                                                       .arg(filename).arg(openAttempts));
+                        close(fd2);
+                        fd2 = -1;
+                        usleep(500000);
+                    }
+                    else
+                    {
+                        lseek(fd2, 0, SEEK_SET);
+                        openAttempts = 0;
+                    }
+                }
+            }
+        }
         else
             remotefile = new RemoteFile(filename);
 
@@ -384,7 +432,7 @@ RingBuffer::RingBuffer(const QString &lfilename, bool write, bool usereadahead)
     smudgeamount = 0;
 }
 
-RingBuffer::RingBuffer(const QString &lfilename, long long size, 
+RingBuffer::RingBuffer(const QString &lfilename, long long size,
                        long long smudge, RemoteEncoder *enc)
 {
     Init();
@@ -398,11 +446,11 @@ RingBuffer::RingBuffer(const QString &lfilename, long long size,
     normalfile = false;
     filename = (QString)lfilename;
     filesize = size;
-   
+
     if (recorder_num == 0)
     {
-        tfw = new ThreadedFileWriter(filename.ascii(), 
-                                     O_WRONLY|O_CREAT|O_TRUNC|O_LARGEFILE, 
+        tfw = new ThreadedFileWriter(filename.ascii(),
+                                     O_WRONLY|O_CREAT|O_TRUNC|O_LARGEFILE,
                                      0644);
         fd2 = open(filename.ascii(), O_RDONLY|O_LARGEFILE|O_STREAMING);
     }
@@ -459,7 +507,7 @@ RingBuffer::~RingBuffer(void)
         delete tfw;
         tfw = NULL;
     }
-    if (fd2 > 0)
+    if (fd2 >= 0)
     {
         close(fd2);
     }
@@ -500,10 +548,10 @@ void RingBuffer::Reset(void)
         if (readaheadrunning)
             ResetReadAhead(0);
     }
-   
+
     numfailures = 0;
     commserror = false;
- 
+
     pthread_rwlock_unlock(&rwlock);
 }
 
@@ -514,7 +562,7 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
     unsigned errcnt = 0;
     unsigned zerocnt = 0;
 
-    while (tot < sz) 
+    while (tot < sz)
     {
         ret = read(fd, (char *)data + tot, sz - tot);
         if (ret < 0)
@@ -525,7 +573,7 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
             perror("ERROR: file I/O problem in 'safe_read()'");
             errcnt++;
             numfailures++;
-            if (errcnt == 3) 
+            if (errcnt == 3)
                 break;
         }
         else if (ret > 0)
@@ -539,8 +587,7 @@ int RingBuffer::safe_read(int fd, void *data, unsigned sz)
                 break;
 
             zerocnt++;
-            if ((normalfile && zerocnt > 15) || 
-                zerocnt >= 50) // 3 second timeout with usleep(60000)
+            if (zerocnt >= 50) // 3 second timeout with usleep(60000)
             {
                 break;
             }
@@ -627,7 +674,7 @@ int RingBuffer::ReadBufFree(void)
 int RingBuffer::ReadBufAvail(void)
 {
     int ret = 0;
-    
+
     readAheadLock.lock();
 
     if (rbwpos >= rbrpos)
@@ -728,7 +775,7 @@ void RingBuffer::ReadAheadThread(void)
     pausereadthread = false;
 
     readAheadBuffer = new char[READ_AHEAD_SIZE + 256000];
-    
+
     ResetReadAhead(0);
     totfree = ReadBufFree();
 
@@ -776,14 +823,14 @@ void RingBuffer::ReadAheadThread(void)
             {
                 if (remotefile)
                 {
-                    ret = safe_read(remotefile, readAheadBuffer + rbwpos, 
+                    ret = safe_read(remotefile, readAheadBuffer + rbwpos,
                                     totfree);
 
                     if (internalreadpos + totfree > filesize)
                     {
                         int toread = filesize - readpos;
                         int left = totfree - toread;
- 
+
                         internalreadpos = left;
                     }
                     else
@@ -798,7 +845,7 @@ void RingBuffer::ReadAheadThread(void)
                     int left = totfree - toread;
                     lseek(fd2, 0, SEEK_SET);
 
-                    ret = safe_read(fd2, readAheadBuffer + rbwpos + toread, 
+                    ret = safe_read(fd2, readAheadBuffer + rbwpos + toread,
                                     left);
                     ret += toread;
 
@@ -834,7 +881,7 @@ void RingBuffer::ReadAheadThread(void)
         }
 
         if (!readsallowed && used >= fill_min)
-            readsallowed = true;        
+            readsallowed = true;
 
         if (readsallowed && used < fill_min && !ateof)
         {
@@ -847,13 +894,13 @@ void RingBuffer::ReadAheadThread(void)
             readsAllowedWait.wakeAll();
 
         availWaitMutex.lock();
-        if (commserror || ateof || stopreads || 
+        if (commserror || ateof || stopreads ||
             (wanttoread <= used && wanttoread > 0))
         {
             availWait.wakeAll();
         }
         availWaitMutex.unlock();
-            
+
         pthread_rwlock_unlock(&rwlock);
 
         if ((used >= fill_threshold || wantseek) && !pausereadthread)
@@ -874,22 +921,36 @@ int RingBuffer::ReadFromBuf(void *buf, int count)
         return 0;
 
     bool readone = false;
-
+    int readErr = 0;
+    
     if (readaheadpaused && stopreads)
     {
         readone = true;
         Unpause();
     }
-    else 
+    else
+    {
         while (!readsallowed && !stopreads)
         {
             if (!readsAllowedWait.wait(5000))
+            {
                  VERBOSE(VB_IMPORTANT, "taking too long to be allowed to read..");
+                 readErr++;
+                 if (readErr > 2)
+                 {
+                     VERBOSE(VB_IMPORTANT, "Took more than 10 seconds to be allowed to read, "
+                                           "aborting.");
+                     wanttoread = 0;
+                     stopreads = true;
+                     return 0;
+                 }
+            }
         }
-
+    }
+    
     int avail = ReadBufAvail();
-
-    int readErr = 0;
+    readErr = 0;
+    
 
     while (avail < count && !stopreads)
     {
@@ -991,7 +1052,7 @@ int RingBuffer::Read(void *buf, int count)
         if (remotefile)
         {
             ret = ReadFromBuf(buf, count);
-  
+
             if (readpos + ret > filesize)
             {
                 int toread = filesize - readpos;
@@ -1019,7 +1080,7 @@ int RingBuffer::Read(void *buf, int count)
                 if (!availWait.wait(&availWaitMutex, 15000))
                 {
                     VERBOSE(VB_IMPORTANT,"Couldn't read data from the capture card in 15 "
-                            "seconds.  Game over, man.");
+                            "seconds. Stopping.");
                     StopReads();
                 }
 
@@ -1038,16 +1099,16 @@ int RingBuffer::Read(void *buf, int count)
                 int toread = filesize - readpos;
 
                 ret = safe_read(fd2, buf, toread);
-  
+
                 int left = count - toread;
                 lseek(fd2, 0, SEEK_SET);
 
                 ret = safe_read(fd2, (char *)buf + toread, left);
                 ret += toread;
- 
+
                 totalreadpos += ret;
                 readpos = left;
-            } 
+            }
             else
             {
                 ret = safe_read(fd2, buf, count);
@@ -1155,7 +1216,7 @@ long long RingBuffer::GetFileWritePosition(void)
 
 long long RingBuffer::Seek(long long pos, int whence)
 {
-    wantseek = true;  
+    wantseek = true;
     pthread_rwlock_wrlock(&rwlock);
     wantseek = false;
 
@@ -1170,7 +1231,7 @@ long long RingBuffer::Seek(long long pos, int whence)
                 ret = lseek(fd2, pos, whence);
             else
             {
-                long long realseek = readpos + pos;        
+                long long realseek = readpos + pos;
                 ret = lseek(fd2, realseek, SEEK_SET);
             }
         }
@@ -1267,7 +1328,7 @@ long long RingBuffer::GetFreeSpaceWithReadChange(long long readchange)
         if (readchange > 0)
             readchange = 0 - (filesize - readchange);
 
-        return GetFreeSpace() + readchange; 
+        return GetFreeSpace() + readchange;
     }
     else
     {
@@ -1290,9 +1351,9 @@ long long RingBuffer::GetWritePosition(void)
     return writepos;
 }
 
-long long RingBuffer::GetTotalWritePosition(void) 
-{ 
-    return totalwritepos; 
+long long RingBuffer::GetTotalWritePosition(void)
+{
+    return totalwritepos;
 }
 
 long long RingBuffer::GetRealFileSize(void)
