@@ -14,7 +14,11 @@
 
 extern "C" {
 #include <inttypes.h>
-#include "ivtv-ext-api.h"
+#ifdef USING_IVTV_HEADER
+#include <linux/ivtv.h>
+#else
+#include "ivtv_myth.h"
+#endif
 }
 
 using namespace std;
@@ -45,6 +49,7 @@ const char* MpegRecorder::aspectRatio[] = { "Square", "4:3", "16:9",
 MpegRecorder::MpegRecorder()
             : RecorderBase()
 {
+    errored = false;
     paused = false;
     mainpaused = false;
     recording = false;
@@ -76,9 +81,9 @@ MpegRecorder::MpegRecorder()
 
 MpegRecorder::~MpegRecorder()
 {
-    if (chanfd > 0)
+    if (chanfd >= 0)
         close(chanfd);
-    if (readfd > 0)
+    if (readfd >= 0)
         close(readfd);
 }
 
@@ -223,26 +228,27 @@ void MpegRecorder::SetOptionsFromProfile(RecordingProfile *profile,
     }
 }
 
-void MpegRecorder::openMpegFileAsInput(void)
+bool MpegRecorder::OpenMpegFileAsInput(void)
 {
     chanfd = readfd = open(videodevice.ascii(), O_RDONLY);
 
-    if (readfd <= 0)
+    if (readfd < 0)
     {
         cerr << "Can't open MPEG File: " << videodevice << endl;
         perror("open mpeg file:");
-        return;
+        return false;
     }
+    return true;
 }
 
-void MpegRecorder::openV4L2DeviceAsInput(void)
+bool MpegRecorder::OpenV4L2DeviceAsInput(void)
 {
     chanfd = open(videodevice.ascii(), O_RDWR);
-    if (chanfd <= 0)
+    if (chanfd < 0)
     {
         cerr << "Can't open video device: " << videodevice << endl;
         perror("open video:");
-        return;
+        return false;
     }
 
     struct v4l2_format vfmt;
@@ -254,7 +260,7 @@ void MpegRecorder::openV4L2DeviceAsInput(void)
     {
         cerr << "Error getting format\n";
         perror("VIDIOC_G_FMT:");
-        return;
+        return false;
     }
 
     vfmt.fmt.pix.width = width;
@@ -264,7 +270,7 @@ void MpegRecorder::openV4L2DeviceAsInput(void)
     {
         cerr << "Error setting format\n";
         perror("VIDIOC_S_FMT:");
-        return;
+        return false;
     }
 
     struct ivtv_ioctl_codec ivtvcodec;
@@ -274,7 +280,7 @@ void MpegRecorder::openV4L2DeviceAsInput(void)
     {
         cerr << "Error getting codec params\n";
         perror("IVTV_IOC_G_CODEC:");
-        return;
+        return false;
     }
 
     // only 48kHz works properly.
@@ -320,14 +326,14 @@ void MpegRecorder::openV4L2DeviceAsInput(void)
         ivtvcodec.bitrate = ivtvcodec.bitrate_peak;
 
     // framerate (1 = 25fps, 0 = 30fps)
-    ivtvcodec.framerate = (!ntsc);
+    ivtvcodec.framerate = (!ntsc_framerate);
     ivtvcodec.stream_type = streamtype;
 
     if (ioctl(chanfd, IVTV_IOC_S_CODEC, &ivtvcodec) < 0)
     {
         cerr << "Error setting codec params\n";
         perror("IVTV_IOC_S_CODEC:");
-        return;
+        return false;
     }
 
     struct v4l2_control ctrl;
@@ -338,32 +344,55 @@ void MpegRecorder::openV4L2DeviceAsInput(void)
     {
         cerr << "Error setting codec params\n";
         perror("VIDIOC_S_CTRL:");
-        return;
+        return false;
+    }
+
+    struct ivtv_sliced_vbi_format vbifmt;
+    if (vbimode) {
+        memset(&vbifmt, 0, sizeof(struct ivtv_sliced_vbi_format));
+        vbifmt.service_set = (1==vbimode) ? VBI_TYPE_TELETEXT : VBI_TYPE_CC;
+        if ((ioctl(chanfd, IVTV_IOC_S_VBI_MODE, &vbifmt) < 0) || 
+            (ioctl(chanfd, IVTV_IOC_S_VBI_EMBED, &vbifmt) < 0)) 
+        {
+            VERBOSE(VB_IMPORTANT, QString("Can't enable VBI recording"));
+            perror("vbi");
+        }
+        ioctl(chanfd, IVTV_IOC_G_VBI_MODE, &vbifmt);
+        VERBOSE(VB_RECORD, QString("VBI service:%1, packet size:%2, io size:%3").
+                arg(vbifmt.service_set).arg(vbifmt.packet_size).arg(vbifmt.io_size));
     }
 
     readfd = open(videodevice.ascii(), O_RDWR);
-    if (readfd <= 0)
+    if (readfd < 0)
     {
         cerr << "Can't open video device: " << videodevice << endl;
         perror("open video:");
-        return;
+        return false;
     }
+    return true;
+}
+
+bool MpegRecorder::Open(void)
+{
+    if (deviceIsMpegFile)
+        return OpenMpegFileAsInput();
+    else
+        return OpenV4L2DeviceAsInput();
 }
 
 void MpegRecorder::StartRecording(void)
 {
-    if (deviceIsMpegFile)
-        openMpegFileAsInput();
-    else
-        openV4L2DeviceAsInput();
-
-    if ((chanfd <= 0) || (readfd <= 0))
-        return;
+    if (!Open())
+    {
+	errored = true;
+	return;
+    }
 
     if (!SetupRecording())
     {
-        cerr << "Error initializing recording\n";
-        return;
+        VERBOSE(VB_IMPORTANT, "Error initializing recording");
+	errored = true;
+	return;
     }
 
     encoding = true;
@@ -385,7 +414,7 @@ void MpegRecorder::StartRecording(void)
             pauseWait.wakeAll();
 
             if ((!deviceIsMpegFile) &&
-                (readfd > 0))
+                (readfd >= 0))
             {
                 close(readfd);
                 readfd = -1;
@@ -415,7 +444,7 @@ void MpegRecorder::StartRecording(void)
             close(readfd);
             readfd = open(videodevice.ascii(), O_RDONLY);
 
-            if (readfd > 0)
+            if (readfd >= 0)
                 ret = read(readfd, buffer, bufferSize);
             if (ret <= 0)
             {
@@ -446,11 +475,12 @@ int MpegRecorder::GetVideoFd(void)
 
 // start common code to the dvbrecorder class.
 
-static void mpg_write_packet(void *opaque, uint8_t *buf, int buf_size)
+static int mpg_write_packet(void *opaque, uint8_t *buf, int buf_size)
 {
     (void)opaque;
     (void)buf;
     (void)buf_size;
+    return 0;
 }
 
 static int mpg_read_packet(void *opaque, uint8_t *buf, int buf_size)
@@ -646,6 +676,7 @@ void MpegRecorder::StopRecording(void)
 
 void MpegRecorder::Reset(void)
 {
+    errored = false;
     AVPacketList *pktl = NULL;
     while ((pktl = ic->packet_buffer))
     {
