@@ -48,7 +48,8 @@ extern "C" {
 
 const int kNumBuffers = 31;
 const int kNeedFreeFrames = 1;
-const int kPrebufferFrames = 12;
+const int kPrebufferFramesNormal = 12;
+const int kPrebufferFramesSmall = 4;
 const int kKeepPrebuffer = 2;
 
 struct XvData
@@ -68,6 +69,7 @@ struct XvData
 
     int colorkey;
     bool needdrawcolor;
+    bool initialdraw;
 };
 
 #define GUID_I420_PLANAR 0x30323449
@@ -195,6 +197,7 @@ void VideoOutputXv::InputChanged(int width, int height, float aspect)
     pauseFrame.buf = new unsigned char[pauseFrame.size];
     pauseFrame.qscale_table = scratchFrame->qscale_table = NULL;
     pauseFrame.qstride = scratchFrame->qstride = 0;
+    pauseFrame.frameNumber = scratchFrame->frameNumber;
 
     pthread_mutex_unlock(&lock);
 }
@@ -225,6 +228,9 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
 {
     pthread_mutex_init(&lock, NULL);
 
+    needrepaint = true;
+    data->initialdraw = true;
+
     int (*old_handler)(Display *, XErrorEvent *);
     int i, ret;
     XJ_caught_error = 0;
@@ -238,7 +244,8 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
     XvAdaptorInfo *ai;
 
     VideoOutput::InitBuffers(kNumBuffers, true, kNeedFreeFrames,
-                             kPrebufferFrames, kKeepPrebuffer);
+                             kPrebufferFramesNormal, kPrebufferFramesSmall, 
+                             kKeepPrebuffer);
     VideoOutput::Init(width, height, aspect, winid, winx, winy, winw, winh, 
                       embedid);
 
@@ -452,14 +459,15 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
             }
         } 
 
-        if (!foundimageformat)
-        {
-            VERBOSE(VB_IMPORTANT, "Couldn't find the proper Xv image format, exiting");
-            exit(-26);
-        }
-
         if (fo)
             XFree(fo);
+
+        if (!foundimageformat)
+        {
+            VERBOSE(VB_IMPORTANT, "Couldn't find the proper Xv image format");
+            errored = true;
+            return false;
+        }
     }
 
     data->XJ_gc = XCreateGC(data->XJ_disp, data->XJ_win, 0, 0);
@@ -497,7 +505,28 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
     pauseFrame.buf = new unsigned char[pauseFrame.size];
     pauseFrame.qscale_table = NULL;
     pauseFrame.qstride = 0;
+    pauseFrame.frameNumber = scratchFrame->frameNumber;
 
+    InitColorKey(true);
+   
+    MoveResize();
+ 
+    if (gContext->GetNumSetting("UseOutputPictureControls", 0))
+    {
+        ChangePictureAttribute(kPictureAttribute_Brightness, brightness);
+        ChangePictureAttribute(kPictureAttribute_Contrast, contrast);
+        ChangePictureAttribute(kPictureAttribute_Colour, colour);
+        ChangePictureAttribute(kPictureAttribute_Hue, hue);
+    }
+
+    XJ_started = true;
+
+    return true;
+}
+
+void VideoOutputXv::InitColorKey(bool turnoffautopaint)
+{
+    int ret = Success;
     data->needdrawcolor = true;
 
     Atom xv_atom;
@@ -513,13 +542,13 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
             {
                 xv_atom = XInternAtom(data->XJ_disp, "XV_AUTOPAINT_COLORKEY",
                                       False);
-                if (xv_atom != None)
-                {
-                    ret = XvSetPortAttribute(data->XJ_disp, xv_port, xv_atom,
-                                             1);
-                    if (ret == Success)
-                        data->needdrawcolor = false;
-                }
+                if (xv_atom == None)
+                    continue;
+
+                if (turnoffautopaint)
+                    XvSetPortAttribute(data->XJ_disp, xv_port, xv_atom, 0);
+                else
+                    XvSetPortAttribute(data->XJ_disp, xv_port, xv_atom, 1);
             }
         }
         XFree(attributes);
@@ -534,27 +563,21 @@ bool VideoOutputXv::Init(int width, int height, float aspect,
                                      &data->colorkey);
             if (ret != Success)
             {
-                cerr << "Couldn't get the color key color, and we need it.\n";
-                cerr << "You likely won't get any video.\n";
+                VERBOSE(VB_IMPORTANT,
+                        "Couldn't get the color key color, and we need it.\n"
+                        "You likely won't get any video.");
                 data->colorkey = 0;
             }
         }
     }
-
-    MoveResize();
-
-    if (gContext->GetNumSetting("UseOutputPictureControls", 0))
-    {
-        ChangePictureAttribute(kPictureAttribute_Brightness, brightness);
-        ChangePictureAttribute(kPictureAttribute_Contrast, contrast);
-        ChangePictureAttribute(kPictureAttribute_Colour, colour);
-        ChangePictureAttribute(kPictureAttribute_Hue, hue);
-    }
-
-    XJ_started = true;
-
-    return true;
 }
+
+bool VideoOutputXv::SetupDeinterlace(bool interlaced)
+{
+    bool deint = VideoOutput::SetupDeinterlace(interlaced);
+    return deint;
+}
+
 bool VideoOutputXv::ApproveDeintFilter(const QString& filtername) const
 {
     if (filtername == "bobdeint")
@@ -697,6 +720,13 @@ void VideoOutputXv::Exit(void)
 {
     if (XJ_started) 
     {
+        XSetForeground(data->XJ_disp, data->XJ_gc, XJ_black);
+        XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc,
+                       dispx, dispy, dispw, disph);
+
+        m_deinterlacing = false;
+        InitColorKey(false);
+
         XJ_started = false;
 
         if ( xv_port != -1 )
@@ -790,7 +820,10 @@ void VideoOutputXv::DeleteXBuffers()
 void VideoOutputXv::EmbedInWidget(WId wid, int x, int y, int w, int h)
 {
     if (embedding)
+    {
+        MoveResize();
         return;
+    }
 
     pthread_mutex_lock(&lock);
     data->XJ_curwin = wid;
@@ -830,9 +863,17 @@ void VideoOutputXv::StopEmbedding(void)
 
 void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 {
+    if (IsErrored())
+    {
+        VERBOSE(VB_IMPORTANT, "VideoOutputXv::PrepareFrame() called while IsErrored is true.");
+        return;
+    }
+
     if (!buffer)
         buffer = scratchFrame;
 
+    framesPlayed = buffer->frameNumber + 1;
+
     if (xv_port != -1)
     {
         pthread_mutex_lock(&lock);
@@ -845,6 +886,13 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
             return;
         }
 
+        if (needrepaint)
+        {
+            data->initialdraw = false;
+            DrawUnusedRects(false);
+            needrepaint = false;
+        }
+
         if (colorid == GUID_YV12_PLANAR)
         {
             int width = buffer->width;
@@ -861,8 +909,9 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 
         int src_x = imgx, src_y = imgy, src_w = imgw, src_h = imgh;
         int dest_y = dispyoff;
+        bool drawn = false;
 
-        if (m_deinterlacing)
+        if (m_deinterlacing && (m_deintfiltername == "bobdeint"))
         {
             if ((t == kScan_Interlaced && buffer->top_field_first == 1) ||
                 (t == kScan_Intr2ndField && buffer->top_field_first == 0))
@@ -870,6 +919,10 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
                 // Show top field
                 src_y = imgy / 2;
                 src_h = imgh / 2;
+                XvShmPutImage(data->XJ_disp, xv_port, data->XJ_curwin,
+                              data->XJ_gc, image, src_x, src_y, src_w, src_h,
+                              dispxoff, dest_y, dispwoff, disphoff, False);
+                drawn = true;
             }
             else if ((t == kScan_Interlaced && buffer->top_field_first == 0) ||
                      (t == kScan_Intr2ndField && buffer->top_field_first == 1))
@@ -877,13 +930,29 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
                 // Show bottom field
                 src_y = (buffer->height + imgy) / 2;
                 src_h = imgh / 2;
-                dest_y++;
+                int halfLineSrc = (int)round(((float)disphoff) / imgh - 0.001f);
+                src_h -= (halfLineSrc) ? 1 : 0;
+
+//                VERBOSE(VB_PLAYBACK,
+//                        QString("XvPut img_h(%1-%6) dest y(%2+%4) h(%3-%5)")
+//                        .arg(imgh).arg(dest_y).arg(disphoff)
+//                        .arg(halfLineSrc).arg(2 * halfLineSrc)
+//                        .arg(2*(halfLineSrc ? 1 : 0))); // 2* for imgh coords
+
+                XvShmPutImage(data->XJ_disp, xv_port, data->XJ_curwin,
+                              data->XJ_gc, image, src_x, src_y, src_w, src_h,
+                              dispxoff, dest_y   + halfLineSrc,
+                              dispwoff, disphoff - 2 * halfLineSrc, False);
+                drawn = true;
             }
         }
 
-        XvShmPutImage(data->XJ_disp, xv_port, data->XJ_curwin, data->XJ_gc,
-                      image, src_x, src_y, src_w, src_h, dispxoff, dest_y,
-                      dispwoff, disphoff, False);
+        if (!drawn)
+        {
+            XvShmPutImage(data->XJ_disp, xv_port, data->XJ_curwin, data->XJ_gc,
+                          image, src_x, src_y, src_w, src_h, dispxoff, dest_y,
+                          dispwoff, disphoff, False);
+        }
 
         pthread_mutex_unlock(&lock);
     }
@@ -947,7 +1016,8 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
             case 32: av_format = PIX_FMT_RGBA32; break;
             default: 
                 VERBOSE(VB_IMPORTANT, "Non Xv mode only supports 16, 24, and 32 bpp displays");
-                exit(-27);
+                errored = true;
+                return;
         }
 
         avpicture_fill(&image_in, (uint8_t *)image->data, 
@@ -973,18 +1043,30 @@ void VideoOutputXv::PrepareFrame(VideoFrame *buffer, FrameScanType t)
 
 void VideoOutputXv::Show(FrameScanType )
 {
+    if (IsErrored())
+    {
+        VERBOSE(VB_IMPORTANT, "VideoOutputXv::Show() called while IsErrored istrue.");
+        return;
+    }
+
     pthread_mutex_lock(&lock);
     XSync(data->XJ_disp, False);
     pthread_mutex_unlock(&lock);
 }
 
-void VideoOutputXv::DrawUnusedRects(void)
+void VideoOutputXv::DrawUnusedRects(bool sync)
 {
-    if (data->needdrawcolor)
+    // boboff assumes the smallest interlaced resolution is 480 lines
+    int boboff = (int)round(((float)disphoff) / 480 - 0.001f);
+    boboff = (m_deinterlacing && m_deintfiltername == "bobdeint") ? boboff : 0;
+    //VERBOSE(VB_PLAYBACK, QString("disphoff(%1) boboff(%2) ndc(%3)")
+    //        .arg(disphoff).arg(boboff).arg((data->needdrawcolor?"yes":"no")));
+
+    if (data->needdrawcolor && !data->initialdraw)
     {
         XSetForeground(data->XJ_disp, data->XJ_gc, data->colorkey);
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, dispx, 
-                       dispy, dispw, disph);
+                       dispy + boboff, dispw, disph - 2 * boboff);
     }
 
     // Draw black in masked areas
@@ -992,19 +1074,21 @@ void VideoOutputXv::DrawUnusedRects(void)
 
     if (dispxoff > dispx) // left
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispx, dispy, dispxoff-dispx, disph);
-    if (dispxoff+dispwoff < dispx+dispw) // right
+                       dispx, dispy, dispxoff - dispx, disph);
+    if (dispxoff + dispwoff < dispx + dispw) // right
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispxoff+dispwoff, dispy, 
-                       (dispx+dispw)-(dispxoff+dispwoff), disph);
-    if (dispyoff > dispy) // bottom
+                       dispxoff + dispwoff, dispy, 
+                       (dispx + dispw) - (dispxoff + dispwoff), disph);
+    if (dispyoff + boboff > dispy) // top of screen
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispx, dispy, dispw, dispyoff-dispy);
-    if (dispyoff+disphoff < dispy+disph) // top
+                       dispx, dispy, dispw, dispyoff + boboff - dispy);
+    if (dispyoff + disphoff < dispy + disph + boboff) // bottom of screen
         XFillRectangle(data->XJ_disp, data->XJ_curwin, data->XJ_gc, 
-                       dispx, dispyoff+disphoff, 
-                       dispw, (dispy+disph)-(dispyoff+disphoff));
-    XSync(data->XJ_disp, false);
+                       dispx, dispyoff + disphoff - boboff, 
+                       dispw, (dispy + disph + boboff) - (dispyoff + disphoff));
+
+    if (sync)
+        XSync(data->XJ_disp, false);
 }
 
 float VideoOutputXv::GetDisplayAspect(void)
@@ -1028,6 +1112,12 @@ void VideoOutputXv::ProcessFrame(VideoFrame *frame, OSD *osd,
                                  FilterChain *filterList,
                                  NuppelVideoPlayer *pipPlayer)
 {
+    if (IsErrored())
+    {
+        VERBOSE(VB_IMPORTANT, "VideoOutputXv::ProcessFrame() called while IsErrored is true.");
+        return;
+    }
+
     pthread_mutex_lock(&lock);
 
     bool pauseframe = false;
