@@ -16,8 +16,6 @@ using namespace std;
 #include "tv.h"
 #include "autoexpire.h"
 #include "scheduler.h"
-#include "transcoder.h"
-#include "commercialflag.h"
 #include "mainserver.h"
 #include "encoderlink.h"
 #include "remoteutil.h"
@@ -26,12 +24,12 @@ using namespace std;
 #include "libmythtv/programinfo.h"
 #include "libmyth/mythcontext.h"
 #include "libmythtv/dbcheck.h"
+#include "libmythtv/jobqueue.h"
 
 QMap<int, EncoderLink *> tvList;
 AutoExpire *expirer = NULL;
 Scheduler *sched = NULL;
-Transcoder *trans = NULL;
-CommercialFlagger *commflag = NULL;
+JobQueue *jobqueue = NULL;
 QString pidfile;
 QString lockfile_location;
 HouseKeeper *housekeeping = NULL;
@@ -156,9 +154,6 @@ void cleanup(void)
     if (sched)
         delete sched;
 
-    if (trans)
-        delete trans;
-
     if (pidfile != "")
         unlink(pidfile.ascii());
 
@@ -179,6 +174,7 @@ int main(int argc, char **argv)
     bool daemonize = false;
     bool printsched = false;
     bool testsched = false;
+    bool resched = false;
     bool nosched = false;
     bool printexpire = false;
     for (int argpos = 1; argpos < a.argc(); ++argpos)
@@ -297,6 +293,11 @@ int main(int argc, char **argv)
                             print_verbose_messages |= VB_COMMFLAG;
                             verboseString += " " + *it;
                         }
+                        else if(!strcmp(*it,"jobqueue"))
+                        {
+                            print_verbose_messages |= VB_JOBQUEUE;
+                            verboseString += " " + *it;
+                        }
                         else if(!strcmp(*it,"audio"))
                         {
                             print_verbose_messages |= VB_AUDIO;
@@ -307,6 +308,11 @@ int main(int argc, char **argv)
                             print_verbose_messages |= VB_LIBAV;
                             verboseString += " " + *it;
                         }
+                        else if(!strcmp(*it,"siparser"))
+                        {
+                            print_verbose_messages |= VB_SIPARSER;
+                            verboseString += " " + *it;
+                        }
                         else
                         {
                             cerr << "Unknown argument for -v/--verbose: "
@@ -329,6 +335,10 @@ int main(int argc, char **argv)
         {
             testsched = true;
         } 
+        else if (!strcmp(a.argv()[argpos],"--resched"))
+        {
+            resched = true;
+        } 
         else if (!strcmp(a.argv()[argpos],"--nosched"))
         {
             nosched = true;
@@ -355,9 +365,11 @@ int main(int argc, char **argv)
                     "-v or --verbose debug-level    Prints more information" << endl <<
                     "                               Accepts any combination (separated by comma)" << endl << 
                     "                               of all,none,quiet,record,playback,channel," << endl <<
-                    "                               osd,file,schedule,network,commflag,audio,libav" << endl <<
+                    "                               osd,file,schedule,network,commflag,audio," << endl <<
+                    "                               libav,jobqueue" << endl <<
                     "--printexpire                  List of auto-expire programs" << endl <<
                     "--printsched                   Upcoming scheduled programs" << endl <<
+                    "--resched                      Force the scheduler to update" << endl <<
                     "--version                      Version information" << endl;
             return -1;
         }
@@ -389,8 +401,6 @@ int main(int argc, char **argv)
         }
     }
 
-    close(0);
-
     if (signal(SIGPIPE, SIG_IGN) == SIG_ERR)
         cerr << "Unable to ignore SIGPIPE\n";
 
@@ -413,9 +423,14 @@ int main(int argc, char **argv)
         // Send stdout and stderr to the logfile
         dup2(logfd, 1);
         dup2(logfd, 2);
+
+        // Close the unduplicated logfd
+        if (logfd != 1 && logfd != 2)
+            close(logfd);
     }
 
-    gContext = new MythContext(MYTH_BINARY_VERSION, false, false);
+    gContext = new MythContext(MYTH_BINARY_VERSION, false);
+    gContext->SetBackend(true);
 
     QSqlDatabase *db = QSqlDatabase::addDatabase("QMYSQL3");
     if (!db)
@@ -445,15 +460,8 @@ int main(int argc, char **argv)
         return -1;
     }
 
-    QSqlDatabase *transthread = QSqlDatabase::addDatabase("QMYSQL3", "TRANSDB");
-    if (!transthread)
-    {
-        cerr << "Couldn't connect to database\n";
-        return -1;
-    }
-
-    QSqlDatabase *flagthread = QSqlDatabase::addDatabase("QMYSQL3", "FLAGDB");
-    if (!flagthread)
+    QSqlDatabase *jobthread = QSqlDatabase::addDatabase("QMYSQL3", "JOBDB");
+    if (!jobthread)
     {
         cerr << "Couldn't connect to database\n";
         return -1;
@@ -466,18 +474,24 @@ int main(int argc, char **argv)
         return -1;
     }
 
-    if (!gContext->OpenDatabase(db) || !gContext->OpenDatabase(subthread) ||
-        !gContext->OpenDatabase(expthread) ||
-        !gContext->OpenDatabase(hkthread) ||
-        !gContext->OpenDatabase(transthread) ||
-        !gContext->OpenDatabase(flagthread) ||
-        !gContext->OpenDatabase(msdb))
+    if (!gContext->OpenDatabase(db, !daemonize && (logfile == "")) ||
+        !gContext->OpenDatabase(subthread, false) ||
+        !gContext->OpenDatabase(expthread, false) ||
+        !gContext->OpenDatabase(hkthread, false) ||
+        !gContext->OpenDatabase(jobthread, false) ||
+        !gContext->OpenDatabase(msdb, false))
     {
         cerr << "Couldn't open database\n";
         return -1;
     }
 
-    UpgradeTVDatabaseSchema();
+    close(0);
+
+    if (!UpgradeTVDatabaseSchema())
+    {
+        VERBOSE(VB_IMPORTANT, "Couldn't upgrade database to new schema");
+        return -1;
+    }    
 
     if (printsched || testsched)
     {
@@ -492,7 +506,7 @@ int main(int argc, char **argv)
             cout << "Calculating Schedule from database.\n" <<
                     "Inputs, Card IDs, and Conflict info may be invalid "
                     "if you have multiple tuners.\n";
-            sched->FillRecordLists();
+            sched->FillRecordListFromDB();
         }
 
         print_verbose_messages |= VB_SCHEDULE;
@@ -501,6 +515,22 @@ int main(int argc, char **argv)
         exit(0);
     }
 
+    if (resched)
+    {
+        gContext->SetBackend(false);
+
+        if (gContext->ConnectToMasterServer())
+        {
+            VERBOSE(VB_IMPORTANT, "Connected to master for reschedule");
+            ScheduledRecording::signalChange(-1);
+        }
+        else
+            VERBOSE(VB_IMPORTANT, "Cannot connect to master for reschedule");
+
+        cleanup();
+        exit(0);
+    }
+
     if (printexpire)
     {
         expirer = new AutoExpire(false, false, db);
@@ -558,11 +588,8 @@ int main(int argc, char **argv)
     QSqlDatabase *hkdb = QSqlDatabase::database("HKDB");
     housekeeping = new HouseKeeper(true, ismaster, hkdb);
 
-    QSqlDatabase *trandb = QSqlDatabase::database("TRANSDB");
-    trans = new Transcoder(&tvList, trandb);
-
-    QSqlDatabase *flagdb = QSqlDatabase::database("FLAGDB");
-    commflag = new CommercialFlagger(ismaster, flagdb);
+    QSqlDatabase *jobdb = QSqlDatabase::database("JOBDB");
+    jobqueue = new JobQueue(ismaster, jobdb);
 
     VERBOSE(VB_ALL, QString("%1 version: %2 www.mythtv.org")
                             .arg(binname).arg(MYTH_BINARY_VERSION));
@@ -605,8 +632,6 @@ int main(int argc, char **argv)
 
     a.exec();
 
-    // delete trans;
-
     gContext->LogEntry("mythbackend", LP_INFO, "MythBackend exiting", "");
     cleanup();
 
