@@ -22,6 +22,7 @@
 using namespace std;
 
 #include "playbackbox.h"
+#include "proglist.h"
 #include "tv.h"
 #include "oldsettings.h"
 #include "NuppelVideoPlayer.h"
@@ -31,6 +32,24 @@ using namespace std;
 #include "programinfo.h"
 #include "scheduledrecording.h"
 #include "remoteutil.h"
+#include "jobqueue.h"
+
+static int comp_programid(ProgramInfo *a, ProgramInfo *b)
+{
+    if (a->programid == b->programid)
+    {
+	if (a->recstartts == b->recstartts)
+	    return 0;
+	if (a->recstartts < b->recstartts)
+	    return -1;
+	else
+	    return 1;
+    }
+    if (a->programid < b->programid)
+        return -1;
+    else
+        return 1;
+}
 
 PlaybackBox::PlaybackBox(BoxType ltype, MythMainWindow *parent, 
                          const char *name)
@@ -53,6 +72,8 @@ PlaybackBox::PlaybackBox(BoxType ltype, MythMainWindow *parent,
     titleList << "";
     progIndex = 0;
     titleIndex = 0;
+    playList.clear();
+    onPlaylist = false;
 
     skipUpdate = false;
     skipCnt = 0;
@@ -73,6 +94,8 @@ PlaybackBox::PlaybackBox(BoxType ltype, MythMainWindow *parent,
     popup = NULL;
     curitem = NULL;
     delitem = NULL;
+ 
+    titleView = true;
 
     if (gContext->GetNumSetting("UseArrowAccels", 1))
         arrowAccel = true;
@@ -108,6 +131,7 @@ PlaybackBox::PlaybackBox(BoxType ltype, MythMainWindow *parent,
     infoRect = QRect(0, 0, 0, 0);
     usageRect = QRect(0, 0, 0, 0);
     videoRect = QRect(0, 0, 0, 0);
+    curGroupRect = QRect(0, 0, 0, 0);
 
     showDateFormat = gContext->GetSetting("ShortDateFormat", "M/d");
     showTimeFormat = gContext->GetSetting("TimeFormat", "h:mm AP");
@@ -156,7 +180,7 @@ PlaybackBox::PlaybackBox(BoxType ltype, MythMainWindow *parent,
     else
         haveGroupInfoSet = false;
 
-
+    
     connected = FillList();
 
     playbackPreview = gContext->GetNumSetting("PlaybackPreview");
@@ -274,6 +298,9 @@ void PlaybackBox::parseContainer(QDomElement &element)
         videoRect = area;
     if (name.lower() == "usage")
         usageRect = area;
+    if (name.lower() == "cur_group")
+        curGroupRect = area;        
+    
 }
 
 void PlaybackBox::parsePopup(QDomElement &element)
@@ -359,14 +386,22 @@ void PlaybackBox::paintEvent(QPaintEvent *e)
     {
         updateShowTitles(&p);
     }
+    
     if (r.intersects(infoRect) && skipUpdate == false)
     {
         updateInfo(&p);
     }
+    
+    if (r.intersects(curGroupRect) && skipUpdate == false)
+    {
+        updateCurGroup(&p);
+    }
+    
     if (r.intersects(usageRect) && skipUpdate == false)
     {
         updateUsage(&p);
     }
+    
     if (r.intersects(videoRect))
     {
         updateVideo(&p);
@@ -382,6 +417,7 @@ void PlaybackBox::paintEvent(QPaintEvent *e)
 
 void PlaybackBox::grayOut(QPainter *tmp)
 {
+    (void)tmp;
 /*
     int transparentFlag = gContext->GetNumSetting("PlayBoxShading", 0);
     if (transparentFlag == 0)
@@ -392,12 +428,36 @@ void PlaybackBox::grayOut(QPainter *tmp)
                         (int)(600*hmult));
 */
 }
-
-void PlaybackBox::updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix)
+void PlaybackBox::updateCurGroup(QPainter *p)
 {
-    if (haveGroupInfoSet)
+    QRect pr = curGroupRect;
+    QPixmap pix(pr.size());
+    pix.fill(this, pr.topLeft());
+    if( recGroup != "Default")
+        updateGroupInfo(p, pr, pix, "cur_group");
+    else
     {
-        LayerSet *container = theme->GetSet("group_info");
+        LayerSet *container = theme->GetSet("cur_group");
+        if (container)
+        {
+            QPainter tmp(&pix);
+            
+            container->ClearAllText();
+            container->Draw(&tmp, 6, 1);
+
+            tmp.end();
+            p->drawPixmap(pr.topLeft(), pix);
+        }
+    
+    }
+}
+
+
+void PlaybackBox::updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix, QString cont_name)
+{
+    LayerSet *container = theme->GetSet(cont_name);
+    if( container)
+    {    
         container->ClearAllText();
         QPainter tmp(&pix);
         QMap<QString, QString> infoMap;
@@ -408,10 +468,14 @@ void PlaybackBox::updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix)
         {
            countInGroup = progLists[""].count(); 
            infoMap["title"] = groupDisplayName;
+           infoMap["group"] = groupDisplayName;
+           infoMap["show"] = QObject::tr("All Programs");
         }
         else
         {                  
             countInGroup = progLists[titleList[titleIndex]].count();
+            infoMap["group"] = groupDisplayName;
+            infoMap["show"] = titleList[titleIndex];
             infoMap["title"] = QString("%1 - %2").arg(groupDisplayName)
                                                  .arg(titleList[titleIndex]);
         }
@@ -426,7 +490,9 @@ void PlaybackBox::updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix)
         else
             infoMap["description"] = QString(tr("There are no recordings in "
                                                 "this display group"));
-        container->ClearAllText();
+        
+        infoMap["rec_count"] = QString("%1").arg(countInGroup);
+                                                
         container->SetText(infoMap);
 
         if (type != Delete)
@@ -438,8 +504,10 @@ void PlaybackBox::updateGroupInfo(QPainter *p, QRect& pr, QPixmap& pix)
         p->drawPixmap(pr.topLeft(), pix);
     }
     else
-        updateProgramInfo(p, pr, pix);
-
+    {
+        if (cont_name == "group_info")
+            updateProgramInfo(p, pr, pix);
+    }
 }
 
 
@@ -884,14 +952,14 @@ void PlaybackBox::updateShowTitles(QPainter *p)
                 tempCurrent = RemoteGetRecordingStatus(tempInfo, overrectime,
                                                      underrectime);
 
-                if (titleList[titleIndex] == "")
+                if ((titleList[titleIndex] == "") || (!(titleView)))
                     tempSubTitle = tempInfo->title; 
                 else
                     tempSubTitle = tempInfo->subtitle;
                 if (tempSubTitle.stripWhiteSpace().length() == 0)
                     tempSubTitle = tempInfo->title;
                 if ((tempInfo->subtitle).stripWhiteSpace().length() > 0 
-                    && titleList[titleIndex] == "")
+                    && ((titleList[titleIndex] == "") || (!(titleView))))
                 {
                     tempSubTitle = tempSubTitle + " - \"" + 
                         tempInfo->subtitle + "\"";
@@ -917,8 +985,14 @@ void PlaybackBox::updateShowTitles(QPainter *p)
                     ltype->EnableForcedFont(cnt, "recording");
                 else if (tempCurrent > 1)
                     ltype->EnableForcedFont(cnt, "recording"); // FIXME: change to overunderrecording, fall back to recording. 
-            }
 
+                QString key;
+                key = tempInfo->chanid + "_" +
+                      tempInfo->startts.toString(Qt::ISODate);
+
+                if (playList.grep(key).count())
+                    ltype->EnableForcedFont(cnt, "tagged");
+            }
         } 
     } 
 
@@ -1064,12 +1138,14 @@ bool PlaybackBox::FillList()
     // Save some information so we can find our place again.
     QString oldtitle = titleList[titleIndex];
     QString oldchanid;
+    QString oldprogramid;
     QDateTime oldstartts;
     p = progLists[oldtitle].at(progIndex);
     if (p)
     {
         oldchanid = p->chanid;
         oldstartts = p->startts;
+        oldprogramid = p->programid;
     }
 
     titleList.clear();
@@ -1080,10 +1156,15 @@ bool PlaybackBox::FillList()
 
     fillRecGroupPasswordCache();
 
+    QMap<QString, QString> sortedList;
+    QRegExp prefixes = tr("^(The |A |An )");
+    QString sTitle = "";
+
     vector<ProgramInfo *> *infoList;
     infoList = RemoteGetRecordedList(listOrder == 0 || type == Delete);
     if (infoList)
     {
+        sortedList[""] = "";
         vector<ProgramInfo *>::iterator i = infoList->begin();
         for ( ; i != infoList->end(); i++)
         {
@@ -1095,8 +1176,28 @@ bool PlaybackBox::FillList()
                  (p->category == recGroup ) &&
                  ( !recGroupPwCache.contains(p->recgroup))))
             {
-                progLists[""].prepend(p);
-                progLists[p->title].prepend(p);
+                if (titleView) // Normal title view 
+                {
+                    progLists[""].prepend(p);
+                    progLists[p->title].prepend(p);
+                    sTitle = p->title;
+                    sTitle.remove(prefixes);
+                    sortedList[sTitle] = p->title;
+                } 
+                else 
+                { 
+                    progLists[""].prepend(p);
+                    progLists[p->recgroup].prepend(p);
+                    sortedList[p->recgroup] = p->recgroup;
+                    // If categories are used as recording groups...
+                    if (gContext->GetNumSetting("UseCategoriesAsRecGroups"))
+                    {
+                        // Recording groups and categories overlap so set flag
+                        progLists[p->category].setAutoDelete(false);
+                        progLists[p->category].prepend(p);
+                        sortedList[p->category] = p->category;
+                    }
+                }
             }
             else
                 delete p;
@@ -1104,20 +1205,49 @@ bool PlaybackBox::FillList()
         delete infoList;
     }
 
-    titleList = progLists.keys();
+    if (sortedList.count() == 0)
+    {
+        progLists[""];
+        titleList << "";
+        progIndex = 0;
+        titleIndex = 0;
+        playList.clear();
+
+        return 0;
+    }
+
+    titleList = sortedList.values();
+
+    QString episodeSort = gContext->GetSetting("PlayBoxEpisodeSort", "Date");
+
+    if (episodeSort == "Id" && titleView)
+    {
+        QMap<QString, ProgramList>::Iterator Iprog;
+        for (Iprog = progLists.begin(); Iprog != progLists.end(); ++Iprog)
+        {
+            if (!Iprog.key().isEmpty())
+                Iprog.data().Sort(comp_programid);
+        }
+    }
 
     // Try to find our old place in the title list.  Scan the new
     // titles backwards until we find where we were or go past.  This
     // is somewhat inefficient, but it works.
+
+    QString oldsTitle = oldtitle;
+    oldsTitle.remove(prefixes);
     titleIndex = titleList.count() - 1;
     for (int i = titleIndex; i >= 0; i--)
     {
-        if (oldtitle > titleList[i])
+        sTitle = titleList[i];
+        sTitle.remove(prefixes);
+        
+        if (oldsTitle > sTitle)
             break;
 
         titleIndex = i;
 
-        if (oldtitle == titleList[i])
+        if (oldsTitle == sTitle)
             break;
     }
 
@@ -1134,7 +1264,12 @@ bool PlaybackBox::FillList()
         {
             p = l->at(i);
 
-            if (listOrder == 0 || type == Delete)
+            if (episodeSort == "Id" && titleView && titleIndex > 0)
+            {
+                if (oldprogramid > p->programid)
+                    break;
+            }
+            else if (listOrder == 0 || type == Delete)
             {
                 if (oldstartts > p->startts)
                     break;
@@ -1242,17 +1377,56 @@ void PlaybackBox::startPlayer(ProgramInfo *rec)
         nvpTimeout.start();
 
         state = kStarting;
-        timer->start(1000 / 30); 
+
+        int previewRate = 30;
+        if (gContext->GetNumSetting("PlaybackPreviewLowCPU", 0))
+        {
+            previewRate = 12;
+        }
+        
+        timer->start(1000 / previewRate); 
     }
 }
 
+void PlaybackBox::playSelectedPlaylist(bool random)
+{
+    state = kStopping;
+
+    if (!curitem)
+        return;
+
+    ProgramInfo *tmpItem;
+    QStringList::Iterator it = playList.begin();
+    QStringList randomList = playList;
+    bool playNext = true;
+    int i = 0;
+   
+    onPlaylist = true;
+    while (randomList.count() && playNext)
+    {
+        if (random)
+            i = (int)(1.0 * randomList.count() * rand() / (RAND_MAX + 1.0));
+
+        it = randomList.at(i);
+        tmpItem = findMatchingProg(*it);
+        if (tmpItem)
+        {
+            ProgramInfo *rec = new ProgramInfo(*tmpItem);
+            playNext = play(rec);
+            delete rec;
+        }
+        randomList.remove(it);
+    }
+    onPlaylist = false;
+}
+
 void PlaybackBox::playSelected()
 {
     state = kStopping;
 
     if (!curitem)
         return;
-     
+
     play(curitem);
 }
 
@@ -1273,7 +1447,22 @@ void PlaybackBox::deleteSelected()
     if (!curitem)
         return;
 
-    remove(curitem);
+    if (!playList.count())
+    {
+        remove(curitem);
+    }
+    else
+    {
+        ProgramInfo *tmpItem;
+        QStringList::Iterator it;
+
+        for (it = playList.begin(); it != playList.end(); ++it )
+        {
+            tmpItem = findMatchingProg(*it);
+            if (tmpItem)
+                remove(tmpItem);
+        }
+    }
 }
 
 void PlaybackBox::expireSelected()
@@ -1286,6 +1475,20 @@ void PlaybackBox::expireSelected()
     expire(curitem);
 }
 
+void PlaybackBox::upcoming()
+{
+    state = kStopping;
+
+    if (!curitem)
+        return;
+
+    ProgLister *pl = new ProgLister(plTitle, curitem->title,
+                                   QSqlDatabase::database(),
+                                   gContext->GetMainWindow(), "proglist");
+    pl->exec();
+    delete pl;
+}
+
 void PlaybackBox::details()
 {
     state = kStopping;
@@ -1311,8 +1514,8 @@ void PlaybackBox::selected()
 
     switch (type) 
     {
-        case Play: play(curitem); break;
-        case Delete: remove(curitem); break;
+        case Play: playSelected(); break;
+        case Delete: deleteSelected(); break;
     }
 }
 
@@ -1343,6 +1546,41 @@ void PlaybackBox::showMenu()
         popup->addButton(tr("Change Group Password"), this,
                          SLOT(showRecGroupPasswordChanger()));
 
+    if (titleView)
+        popup->addButton(tr("Show group list as recording groups"), this,
+                         SLOT(toggleTitleView()));
+    else
+        popup->addButton(tr("Show group list as titles"), this,
+                         SLOT(toggleTitleView()));
+
+    if (playList.count())
+    {
+        popup->addButton(tr("Playlist options"), this,
+                         SLOT(showPlaylistPopup()));
+    }
+    else
+    {
+        if (inTitle)
+        {
+            if (titleView)
+            {
+                popup->addButton(
+                    tr("Add this Category/Title Group to Playlist"), this,
+                       SLOT(togglePlayListTitle()));
+            }
+            else 
+            {
+                popup->addButton(tr("Add this Recording Group to Playlist"),
+                                 this, SLOT(togglePlayListTitle()));
+            }
+        }
+        else 
+        {
+            popup->addButton(tr("Add this recording to Playlist"), this,
+                             SLOT(togglePlayListItem()));
+        }
+    }
+
     popup->addButton(tr("Cancel"), this, SLOT(doCancel()));
 
     popup->ShowPopup(this, SLOT(doCancel()));
@@ -1363,15 +1601,17 @@ void PlaybackBox::showActionsSelected()
     showActions(curitem);
 }
 
-void PlaybackBox::play(ProgramInfo *rec)
+bool PlaybackBox::play(ProgramInfo *rec)
 {
+    bool playCompleted = false;
+
     if (!rec)
-        return;
+        return false;
 
     if (fileExists(rec) == false)
     {
         cerr << "Error: " << rec->pathname << " file not found\n";
-        return;
+        return false;
     }
 
     ProgramInfo *tvrec = new ProgramInfo(*rec);
@@ -1389,7 +1629,7 @@ void PlaybackBox::play(ProgramInfo *rec)
         {
             qApp->unlock();
             qApp->processEvents();
-            usleep(10000);
+            usleep(100000);
             qApp->lock();
         }
     }
@@ -1403,11 +1643,15 @@ void PlaybackBox::play(ProgramInfo *rec)
     bool doremove = false;
     bool doprompt = false;
 
+    if (tv->getEndOfRecording())
+        playCompleted = true;
+
     if (tv->getRequestDelete())
     {
         doremove = true;
     }
     else if (tv->getEndOfRecording() &&
+             !onPlaylist &&
              gContext->GetNumSetting("EndOfRecordingExitPrompt"))
     {
         doprompt = true;
@@ -1429,6 +1673,8 @@ void PlaybackBox::play(ProgramInfo *rec)
     connected = FillList();
     skipUpdate = false;
     update(fullRect);
+
+    return playCompleted;
 }
 
 void PlaybackBox::stop(ProgramInfo *rec)
@@ -1439,10 +1685,7 @@ void PlaybackBox::stop(ProgramInfo *rec)
 void PlaybackBox::doRemove(ProgramInfo *rec, bool forgetHistory)
 {
     RemoteDeleteRecording(rec, forgetHistory);
-    ScheduledRecording::signalChange(QSqlDatabase::database());
-
     connected = FillList();
-
     update(fullRect);
 }
 
@@ -1512,7 +1755,7 @@ void PlaybackBox::showDeletePopup(ProgramInfo *program, deletePopupType types)
     if ((types == EndOfRecording || types == DeleteRecording) &&
         (program->IsSameProgram(*program)))
     {
-        tmpmessage = tr("Yes, but allow future recordings of this episode"); 
+        tmpmessage = tr("Yes, and allow re-record"); 
         tmpslot = SLOT(doDeleteForgetHistory());
         popup->addButton(tmpmessage, this, tmpslot);
     }
@@ -1572,6 +1815,196 @@ void PlaybackBox::showDeletePopup(ProgramInfo *program, deletePopupType types)
     expectingPopup = true;
 }
 
+void PlaybackBox::showPlaylistPopup()
+{
+    if (expectingPopup)
+       cancelPopup();
+
+    backup.begin(this);
+    grayOut(&backup);
+    backup.end();
+
+    popup = new MythPopupBox(gContext->GetMainWindow(), graphicPopup,
+                             popupForeground, popupBackground,
+                             popupHighlight, "playlist popup");
+
+    QLabel *tlabel = NULL;
+    if (playList.count() > 1)
+    {
+        tlabel = popup->addLabel(tr("There are %1 items in the playlist.")
+                                       .arg(playList.count()));
+    } else {
+        tlabel = popup->addLabel(tr("There is %1 item in the playlist.")
+                                       .arg(playList.count()));
+    }
+    tlabel->setAlignment(Qt::AlignCenter | Qt::WordBreak);
+
+    QButton *playButton = popup->addButton(tr("Play"), this, SLOT(doPlayList()));
+    
+    popup->addButton(tr("Shuffle Play"), this, SLOT(doPlayListRandom()));
+    popup->addButton(tr("Clear Playlist"), this, SLOT(doClearPlaylist()));
+ 
+    if (inTitle)
+    {
+        if (titleView)
+            popup->addButton(tr("Toggle playlist for this Category/Title"),
+                             this, SLOT(togglePlayListTitle()));
+        else 
+            popup->addButton(tr("Toggle playlist for this Recording Group"),
+                             this, SLOT(togglePlayListTitle()));
+    }
+    else
+    {
+        popup->addButton(tr("Toggle playlist for this recording"), this,
+                         SLOT(togglePlayListItem()));
+    }
+
+    QLabel *label = popup->addLabel(
+                        tr("These actions affect all items in the playlist"));
+    label->setAlignment(Qt::AlignCenter | Qt::WordBreak);
+
+    popup->addButton(tr("Change Recording Group"), this,
+                     SLOT(doPlaylistChangeRecGroup()));
+    popup->addButton(tr("Delete"), this, SLOT(doPlaylistDelete()));
+
+    playButton->setFocus();
+
+    popup->ShowPopup(this, SLOT(doCancel()));
+  
+    expectingPopup = true;
+}
+
+void PlaybackBox::showPlayFromPopup()
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    backup.begin(this);
+    grayOut(&backup);
+    backup.end();
+
+    popup = new MythPopupBox(gContext->GetMainWindow(), graphicPopup,
+                             popupForeground, popupBackground,
+                             popupHighlight, "playfrom popup");
+
+    initPopup(popup, delitem, "", "");
+
+    QButton *playButton = popup->addButton(tr("Play from bookmark"), this,
+            SLOT(doPlay()));
+    popup->addButton(tr("Play from beginning"), this, SLOT(doPlayFromBeg()));
+     
+    popup->ShowPopup(this, SLOT(doCancel()));
+    playButton->setFocus();
+    
+    expectingPopup = true;
+}
+
+void PlaybackBox::showStoragePopup()
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    backup.begin(this);
+    grayOut(&backup);
+    backup.end();
+
+    popup = new MythPopupBox(gContext->GetMainWindow(), graphicPopup,
+                             popupForeground, popupBackground,
+                             popupHighlight, "storage popup");
+
+    initPopup(popup, delitem, "", tr("A preserved episode is ignored in calculations for deleting episodes above the limit.  Auto-expiration is used to remove eligable programs when disk space is low."));
+
+    QSqlDatabase *db = QSqlDatabase::database();
+
+    QButton *storageButton;
+
+    if (delitem && delitem->GetAutoExpireFromRecorded(db))
+        storageButton = popup->addButton(tr("Don't Auto Expire"), this, SLOT(noAutoExpire()));
+    else
+        storageButton = popup->addButton(tr("Auto Expire"), this, SLOT(doAutoExpire()));
+
+    if (delitem && delitem->UsesMaxEpisodes(db))
+    {
+        if (delitem && delitem->GetPreserveEpisodeFromRecorded(db))
+            popup->addButton(tr("Do not preserve this episode"), this, SLOT(noPreserveEpisode()));
+        else
+            popup->addButton(tr("Preserve this episode"), this, SLOT(doPreserveEpisode()));
+    }
+
+    popup->ShowPopup(this, SLOT(doCancel()));
+    storageButton->setFocus();
+
+    expectingPopup = true;
+}
+
+void PlaybackBox::showRecordingPopup()
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    backup.begin(this);
+    grayOut(&backup);
+    backup.end();
+
+    popup = new MythPopupBox(gContext->GetMainWindow(), graphicPopup,
+                             popupForeground, popupBackground,
+                             popupHighlight, "recording popup");
+
+    initPopup(popup, delitem, "", "");
+
+    QButton *changeButton = popup->addButton(tr("Change Recording Group"), this,
+                     SLOT(showRecGroupChanger()));
+
+    popup->addButton(tr("Edit Recording Schedule"), this,
+                     SLOT(doEditScheduled()));
+    
+    popup->ShowPopup(this, SLOT(doCancel()));
+    changeButton->setFocus();
+    
+    expectingPopup = true;
+}
+
+void PlaybackBox::showJobPopup()
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    backup.begin(this);
+    grayOut(&backup);
+    backup.end();
+
+    popup = new MythPopupBox(gContext->GetMainWindow(), graphicPopup,
+                             popupForeground, popupBackground,
+                             popupHighlight, "job popup");
+
+    initPopup(popup, delitem, "", "");
+
+    QSqlDatabase *db = QSqlDatabase::database();
+
+    QButton *jobButton;
+
+    if (JobQueue::IsJobRunning(db, JOB_TRANSCODE, curitem->chanid,
+                                                  curitem->startts))
+        jobButton = popup->addButton(tr("Stop Transcoding"), this,
+                         SLOT(doBeginTranscoding()));
+    else
+        jobButton = popup->addButton(tr("Begin Transcoding"), this,
+                         SLOT(doBeginTranscoding()));
+
+    if (JobQueue::IsJobRunning(db, JOB_COMMFLAG, curitem->chanid,
+                                                  curitem->startts))
+        popup->addButton(tr("Stop Commercial Flagging"), this,
+                         SLOT(doBeginFlagging()));
+    else
+        popup->addButton(tr("Begin Commercial Flagging"), this,
+                         SLOT(doBeginFlagging()));
+
+    popup->ShowPopup(this, SLOT(doCancel()));
+    jobButton->setFocus();
+    
+    expectingPopup = true;
+}
+
 void PlaybackBox::showActionPopup(ProgramInfo *program)
 {
     backup.begin(this);
@@ -1585,45 +2018,48 @@ void PlaybackBox::showActionPopup(ProgramInfo *program)
     initPopup(popup, program, "", "");
 
     QSqlDatabase *db = QSqlDatabase::database();
+    QButton *playButton;
 
-    QButton *playButton = popup->addButton(tr("Play"), this, SLOT(doPlay()));
+    if (curitem->programflags & FL_BOOKMARK)
+    {
+        playButton = popup->addButton(tr("Play from..."), this,
+                                      SLOT(showPlayFromPopup()));
+    }
+    else
+    {
+        playButton = popup->addButton(tr("Play"), this, SLOT(doPlay()));
+    }
 
+    QString key;
+    key = curitem->chanid + "_" +
+         curitem->startts.toString(Qt::ISODate);
+
+    if (playList.grep(key).count())
+        popup->addButton(tr("Remove from Playlist"), this,
+                         SLOT(togglePlayListItem()));
+    else
+        popup->addButton(tr("Add to Playlist"), this,
+                         SLOT(togglePlayListItem()));
 
     if (RemoteGetRecordingStatus(program, overrectime, underrectime) > 0)
         popup->addButton(tr("Stop Recording"), this, SLOT(askStop()));
 
-    if (delitem && delitem->GetAutoExpireFromRecorded(db))
-        popup->addButton(tr("Don't Auto Expire"), this, SLOT(noAutoExpire()));
-    else
-        popup->addButton(tr("Auto Expire"), this, SLOT(doAutoExpire()));
+    // Remove this check and the auto expire buttons if a third button is added
+    // to the StoragePopup screen.  Otherwise for non-max-episode schedules,
+    // the popup will only show one button
+    if (delitem && delitem->UsesMaxEpisodes(db))
+    {
+        popup->addButton(tr("Storage Options"), this, SLOT(showStoragePopup()));
+    } else {
+        if (delitem && delitem->GetAutoExpireFromRecorded(db))
+            popup->addButton(tr("Don't Auto Expire"), this,
+                             SLOT(noAutoExpire()));
+        else
+            popup->addButton(tr("Auto Expire"), this, SLOT(doAutoExpire()));
+    }
 
-    popup->addButton(tr("Change Recording Group"), this,
-                     SLOT(showRecGroupChanger()));
-
-    popup->addButton(tr("Edit Recording Schedule"), this,
-                     SLOT(doEditScheduled()));
-
-    QString query = QString("SELECT * FROM transcoding WHERE "
-                            "chanid = '%1' AND starttime = '%2';")
-                           .arg(curitem->chanid)
-                           .arg(curitem->startts.toString("yyyyMMddhhmmss"));
-
-    MythContext::KickDatabase(db);
-    QSqlQuery result = db->exec(query);
-
-    if (result.isActive() && result.numRowsAffected() > 0)
-        popup->addButton(tr("Stop Transcoding"), this,
-                         SLOT(doBeginTranscoding()));
-    else
-        popup->addButton(tr("Begin Transcoding"), this,
-                         SLOT(doBeginTranscoding()));
-
-    if (curitem->IsCommProcessing(db))
-        popup->addButton(tr("Stop Commercial Flagging"), this,
-                         SLOT(doBeginFlagging()));
-    else
-        popup->addButton(tr("Begin Commercial Flagging"), this,
-                         SLOT(doBeginFlagging()));
+    popup->addButton(tr("Recording Options"), this, SLOT(showRecordingPopup()));
+    popup->addButton(tr("Job Options"), this, SLOT(showJobPopup()));
 
     popup->addButton(tr("Delete"), this, SLOT(askDelete()));
 
@@ -1684,6 +2120,14 @@ void PlaybackBox::cancelPopup(void)
     setActiveWindow();
 }
 
+void PlaybackBox::doClearPlaylist(void)
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    playList.clear();
+}
+
 void PlaybackBox::doPlay(void)
 {
     if (!expectingPopup)
@@ -1691,7 +2135,61 @@ void PlaybackBox::doPlay(void)
 
     cancelPopup();
 
-    play(delitem);
+    if (curitem)
+        delete curitem;
+
+    curitem = new ProgramInfo(*delitem);
+
+    playSelected();
+}
+
+void PlaybackBox::doPlayFromBeg(void)
+{
+    delitem->setIgnoreBookmark(true);
+    doPlay();
+}
+
+void PlaybackBox::doPlayList(void)
+{
+    if (!expectingPopup)
+        return;
+
+    cancelPopup();
+
+    playSelectedPlaylist(false);
+}
+
+
+void PlaybackBox::doPlayListRandom(void)
+{
+    if (!expectingPopup)
+        return;
+
+    cancelPopup();
+
+    playSelectedPlaylist(true);
+}
+
+void PlaybackBox::doPreserveEpisode(void)
+{
+    if (!expectingPopup)
+        return;
+
+    cancelPopup();
+
+    QSqlDatabase *db = QSqlDatabase::database();
+    delitem->SetPreserveEpisode(true, db);
+}
+
+void PlaybackBox::noPreserveEpisode(void)
+{
+    if (!expectingPopup)
+        return;
+
+    cancelPopup();
+
+    QSqlDatabase *db = QSqlDatabase::database();
+    delitem->SetPreserveEpisode(false, db);
 }
 
 void PlaybackBox::askStop(void)
@@ -1755,22 +2253,19 @@ void PlaybackBox::doBeginTranscoding()
     cancelPopup();
 
     QSqlDatabase *db = QSqlDatabase::database();
+    if (JobQueue::IsJobRunning(db, JOB_TRANSCODE,
+                               curitem->chanid, curitem->startts))
+    {
+        JobQueue::ChangeJobCmds(db, JOB_TRANSCODE,
+                                curitem->chanid, curitem->startts, JOB_STOP);
+    } else {
+        QString jobHost = "";
+        if (gContext->GetNumSetting("JobsRunOnRecordHost", 0))
+            jobHost = gContext->GetHostName();
 
-    QString query = QString("SELECT * FROM transcoding WHERE "
-                            "chanid = '%1' AND starttime = '%2';")
-                           .arg(curitem->chanid)
-                           .arg(curitem->startts.toString("yyyyMMddhhmmss"));
-
-    MythContext::KickDatabase(db);
-
-    QSqlQuery result = db->exec(query);
-
-    if (result.isActive() && result.numRowsAffected() > 0)
-        RemoteQueueTranscode(curitem, TRANSCODE_STOP);
-    else
-        RemoteQueueTranscode(curitem, TRANSCODE_QUEUED |
-                                      TRANSCODE_USE_CUTLIST);
-
+        JobQueue::QueueJob(db, JOB_TRANSCODE, curitem->chanid, curitem->startts,
+                           jobHost, "", "", JOB_USE_CUTLIST);
+    }
 }
 
 void PlaybackBox::doBeginFlagging()
@@ -1785,17 +2280,13 @@ void PlaybackBox::doBeginFlagging()
     QSqlDatabase *db = QSqlDatabase::database();
 
     ProgramInfo *tmpItem = findMatchingProg(curitem);
-    QString detectionHost = gContext->GetSetting("CommercialSkipHost");
 
-    if (detectionHost == "Default")
-        detectionHost = "master";
-
-    if (curitem->IsCommProcessing(db))
+    if (JobQueue::IsJobRunning(db, JOB_COMMFLAG, curitem->chanid,
+                                                  curitem->startts))
     {
-        message = QString("GLOBAL_COMMFLAG STOP %1 %2 %3")
-                          .arg(curitem->chanid)
-                          .arg(curitem->startts.toString(Qt::ISODate))
-                          .arg(detectionHost);
+        JobQueue::ChangeJobCmds(db, JOB_COMMFLAG,
+                                curitem->chanid, curitem->startts, JOB_STOP);
+
         if (tmpItem)
         {
             tmpItem->programflags &= ~FL_EDITING;
@@ -1804,18 +2295,13 @@ void PlaybackBox::doBeginFlagging()
     }
     else
     {
-        message = QString("GLOBAL_COMMFLAG START %1 %2 %3")
-                          .arg(curitem->chanid)
-                          .arg(curitem->startts.toString(Qt::ISODate))
-                          .arg(detectionHost);
-        if (tmpItem)
-        {
-            tmpItem->programflags |= FL_EDITING;
-            tmpItem->programflags &= ~FL_COMMFLAG;
-        }
-    }
+        QString jobHost = "";
+        if (gContext->GetNumSetting("JobsRunOnRecordHost", 0))
+            jobHost = tmpItem->hostname;
 
-    RemoteSendMessage(message);
+        JobQueue::QueueJob(db, JOB_COMMFLAG, curitem->chanid, curitem->startts,
+                           jobHost);
+    }
 
     update(listRect);
 }
@@ -1842,6 +2328,28 @@ void PlaybackBox::noDelete(void)
     timer->start(500);
 }
 
+void PlaybackBox::doPlaylistDelete(void)
+{
+    if (!expectingPopup)
+        return;
+
+    cancelPopup();
+
+    ProgramInfo *tmpItem;
+    QStringList::Iterator it;
+
+    for (it = playList.begin(); it != playList.end(); ++it )
+    {
+        tmpItem = findMatchingProg(*it);
+        if (tmpItem)
+            RemoteDeleteRecording(tmpItem, false);
+    }
+
+    connected = FillList();
+    update(fullRect);
+    playList.clear();
+}
+
 void PlaybackBox::doDelete(void)
 {
     if (!expectingPopup)
@@ -1885,6 +2393,36 @@ ProgramInfo *PlaybackBox::findMatchingProg(ProgramInfo *pginfo)
     return NULL;
 }
 
+ProgramInfo *PlaybackBox::findMatchingProg(QString key)
+{
+    QStringList keyParts;
+
+    if ((key == "") || (key.find('_') < 0))
+        return NULL;
+
+    keyParts = QStringList::split("_", key);
+
+    if (keyParts.count() == 2)
+        return findMatchingProg(keyParts[0], keyParts[1]);
+    else
+        return NULL;
+}
+
+ProgramInfo *PlaybackBox::findMatchingProg(QString chanid, QString startts)
+{
+    ProgramInfo *p;
+    ProgramList *l = &progLists[""];
+
+    for (p = l->first(); p; p = l->next())
+    {
+        if (p->startts.toString(Qt::ISODate) == startts &&
+            p->chanid == chanid)
+            return p;
+    }
+
+    return NULL;
+}
+
 void PlaybackBox::noAutoExpire(void)
 {
     if (!expectingPopup && delitem)
@@ -1939,6 +2477,20 @@ void PlaybackBox::doCancel(void)
     state = kChanging;
 }
 
+void PlaybackBox::toggleTitleView(void)
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    if (titleView) titleView = false;
+    else titleView = true;
+
+    playList.clear();
+    connected = FillList();      
+    skipUpdate = false;
+    update(fullRect);
+}
+
 void PlaybackBox::promptEndOfRecording(ProgramInfo *rec)
 {
     if (!rec)
@@ -1961,6 +2513,74 @@ void PlaybackBox::UpdateProgressBar(void)
     update(usageRect);
 }
 
+void PlaybackBox::togglePlayListTitle(void)
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    if (!curitem)
+        return;
+
+    QString currentTitle = titleList[titleIndex];
+    ProgramInfo *p;
+
+    for( unsigned int i = 0; i < progLists[currentTitle].count(); i++)
+    {
+        p = progLists[currentTitle].at(i);
+        if (p)
+            togglePlayListItem(p);
+    }
+
+    if (inTitle)
+        cursorRight();
+}
+
+void PlaybackBox::togglePlayListItem(void)
+{
+    if (expectingPopup)
+        cancelPopup();
+
+    if (!curitem)
+        return;
+
+    togglePlayListItem(curitem);
+
+    if (!inTitle)
+        cursorDown();
+
+    skipUpdate = false;
+    update(fullRect);
+}
+
+void PlaybackBox::togglePlayListItem(ProgramInfo *pginfo)
+{
+    QString key;
+
+    if (!pginfo)
+        return;
+
+    key  = pginfo->chanid + "_" + pginfo->startts.toString(Qt::ISODate);
+
+    if (playList.grep(key).count())
+    {
+        QStringList tmpList;
+        QStringList::Iterator it;
+
+        tmpList = playList;
+        playList.clear();
+
+        for (it = tmpList.begin(); it != tmpList.end(); ++it )
+        {
+            if (*it != key)
+                playList << *it;
+        }
+    }
+    else
+    {
+        playList << key;
+    }
+}
+
 void PlaybackBox::timeout(void)
 {
     if (titleList.count() <= 1)
@@ -1987,6 +2607,28 @@ void PlaybackBox::keyPressEvent(QKeyEvent *e)
             showIconHelp();
         else if (action == "MENU")
             showMenu();
+        else if (action == "NEXTFAV")
+        {
+            if (inTitle)
+                togglePlayListTitle();
+            else
+                togglePlayListItem();
+        }
+        else if (action == "TOGGLEFAV")
+        {
+            playList.clear();
+            connected = FillList();      
+            skipUpdate = false;
+            update(fullRect);
+        }
+        else if (action == "TOGGLERECORD")
+        {
+            if (titleView) titleView = false;
+            else titleView = true;
+            connected = FillList();
+            skipUpdate = false;
+            update(fullRect);
+        }
         else if (titleList.count() > 1)
         {
             if (action == "DELETE")
@@ -1997,6 +2639,8 @@ void PlaybackBox::keyPressEvent(QKeyEvent *e)
                 showActionsSelected();
             else if (action == "DETAILS")
                 details();
+            else if (action == "UPCOMING")
+                upcoming();
             else if (action == "SELECT")
                 selected();
             else if (action == "UP")
@@ -2354,16 +2998,18 @@ void PlaybackBox::showRecGroupChooser(void)
 
     if (query.isActive() && query.numRowsAffected() > 0)
         while (query.next())
-            if (query.value(0).toString() != recGroup)
-            {
-                QString key = query.value(0).toString();
+        {
+            QString key = QString::fromUtf8(query.value(0).toString());
 
+            if (key != recGroup)
+            {
                 if (key == "Default")
                     groups += tr("Default");
                 else
                     groups += key;
                 recGroupType[key] = "recgroup";
             }
+        }
 
     if (gContext->GetNumSetting("UseCategoriesAsRecGroups"))
     {
@@ -2372,14 +3018,16 @@ void PlaybackBox::showRecGroupChooser(void)
 
         if (query.isActive() && query.numRowsAffected() > 0)
             while (query.next())
-                if ((query.value(0).toString() != recGroup) &&
-                    (query.value(0).toString() != ""))
+            {
+                QString key = QString::fromUtf8(query.value(0).toString());
+
+                if ((key != recGroup) && (key != ""))
                 {
-                    QString key = query.value(0).toString();
                     groups += key;
                     if ( !recGroupType.contains(key))
                         recGroupType[key] = "category";
                 }
+            }
     }
 
     if (recGroup != "All Programs")
@@ -2478,6 +3126,7 @@ void PlaybackBox::chooseSetViewGroup(void)
     inTitle = gContext->GetNumSetting("PlaybackBoxStartInTitle", 0);
     titleIndex = 0;
     progIndex = 0;
+    playList.clear();
 
     connected = FillList();
     skipUpdate = false;
@@ -2543,6 +3192,17 @@ void PlaybackBox::fillRecGroupPasswordCache(void)
                 query.value(1).toString();
 }
 
+void PlaybackBox::doPlaylistChangeRecGroup(void)
+{
+    // This is needed to signal the RecGroup changer that we are taking
+    // action on the entire playlist.  Since the same function can be
+    // called with a playlist built but targeted against a single item
+    // checking the existance of a playlist is not enough
+    onPlaylist = true;
+    showRecGroupChanger();
+    onPlaylist = false;
+}
+
 void PlaybackBox::showRecGroupChanger(void)
 {
     if (!expectingPopup)
@@ -2632,7 +3292,22 @@ void PlaybackBox::changeSetRecGroup(void)
         newRecGroup = "Default";
 
     QSqlDatabase *m_db = QSqlDatabase::database();
-    delitem->ApplyRecordRecGroupChange(m_db, newRecGroup);
+
+    if (onPlaylist)
+    {
+        ProgramInfo *tmpItem;
+        QStringList::Iterator it;
+
+        for (it = playList.begin(); it != playList.end(); ++it )
+        {
+            tmpItem = findMatchingProg(*it);
+            if (tmpItem)
+                tmpItem->ApplyRecordRecGroupChange(m_db, newRecGroup);
+        }
+        playList.clear();
+    } else {
+        delitem->ApplyRecordRecGroupChange(m_db, newRecGroup);
+    }
     
     inTitle = gContext->GetNumSetting("PlaybackBoxStartInTitle", 0);
     titleIndex = 0;
