@@ -30,6 +30,8 @@ class StatusBox : public MythDialog
     void doTunerStatus();
     void doDVBStatus();
     void doLogEntries();
+    void doJobQueueStatus();
+    void doMachineStatus();
     void clicked();
     void setHelpText();
 
@@ -52,9 +54,12 @@ class StatusBox : public MythDialog
 
     QMap<int, QString> contentLines;
     QMap<int, QString> contentDetail;
+    QMap<int, QString> contentFont;
     QMap<int, QString> contentData;
 
     MythMainWindow *my_parent;
+
+    bool isBackend;
 };
 
 #endif
